
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

#include "config.h"

#include <pango.h>
/* enumerations from "pango-attributes.h" */
GType
pango_attr_type_get_type (void)
{
  static gsize g_define_type_id__volatile = 0;

  if (g_once_init_enter (&g_define_type_id__volatile))
    {
      static const GEnumValue values[] = {
        { PANGO_ATTR_INVALID, "PANGO_ATTR_INVALID", "invalid" },
        { PANGO_ATTR_LANGUAGE, "PANGO_ATTR_LANGUAGE", "language" },
        { PANGO_ATTR_FAMILY, "PANGO_ATTR_FAMILY", "family" },
        { PANGO_ATTR_STYLE, "PANGO_ATTR_STYLE", "style" },
        { PANGO_ATTR_WEIGHT, "PANGO_ATTR_WEIGHT", "weight" },
        { PANGO_ATTR_VARIANT, "PANGO_ATTR_VARIANT", "variant" },
        { PANGO_ATTR_STRETCH, "PANGO_ATTR_STRETCH", "stretch" },
        { PANGO_ATTR_SIZE, "PANGO_ATTR_SIZE", "size" },
        { PANGO_ATTR_FONT_DESC, "PANGO_ATTR_FONT_DESC", "font-desc" },
        { PANGO_ATTR_FOREGROUND, "PANGO_ATTR_FOREGROUND", "foreground" },
        { PANGO_ATTR_BACKGROUND, "PANGO_ATTR_BACKGROUND", "background" },
        { PANGO_ATTR_UNDERLINE, "PANGO_ATTR_UNDERLINE", "underline" },
        { PANGO_ATTR_STRIKETHROUGH, "PANGO_ATTR_STRIKETHROUGH", "strikethrough" },
        { PANGO_ATTR_RISE, "PANGO_ATTR_RISE", "rise" },
        { PANGO_ATTR_SHAPE, "PANGO_ATTR_SHAPE", "shape" },
        { PANGO_ATTR_SCALE, "PANGO_ATTR_SCALE", "scale" },
        { PANGO_ATTR_FALLBACK, "PANGO_ATTR_FALLBACK", "fallback" },
        { PANGO_ATTR_LETTER_SPACING, "PANGO_ATTR_LETTER_SPACING", "letter-spacing" },
        { PANGO_ATTR_UNDERLINE_COLOR, "PANGO_ATTR_UNDERLINE_COLOR", "underline-color" },
        { PANGO_ATTR_STRIKETHROUGH_COLOR, "PANGO_ATTR_STRIKETHROUGH_COLOR", "strikethrough-color" },
        { PANGO_ATTR_ABSOLUTE_SIZE, "PANGO_ATTR_ABSOLUTE_SIZE", "absolute-size" },
        { PANGO_ATTR_GRAVITY, "PANGO_ATTR_GRAVITY", "gravity" },
        { PANGO_ATTR_GRAVITY_HINT, "PANGO_ATTR_GRAVITY_HINT", "gravity-hint" },
        { PANGO_ATTR_FONT_FEATURES, "PANGO_ATTR_FONT_FEATURES", "font-features" },
        { PANGO_ATTR_FOREGROUND_ALPHA, "PANGO_ATTR_FOREGROUND_ALPHA", "foreground-alpha" },
        { PANGO_ATTR_BACKGROUND_ALPHA, "PANGO_ATTR_BACKGROUND_ALPHA", "background-alpha" },
        { PANGO_ATTR_ALLOW_BREAKS, "PANGO_ATTR_ALLOW_BREAKS", "allow-breaks" },
        { PANGO_ATTR_SHOW, "PANGO_ATTR_SHOW", "show" },
        { PANGO_ATTR_INSERT_HYPHENS, "PANGO_ATTR_INSERT_HYPHENS", "insert-hyphens" },
        { PANGO_ATTR_OVERLINE, "PANGO_ATTR_OVERLINE", "overline" },
        { PANGO_ATTR_OVERLINE_COLOR, "PANGO_ATTR_OVERLINE_COLOR", "overline-color" },
        { PANGO_ATTR_LINE_HEIGHT, "PANGO_ATTR_LINE_HEIGHT", "line-height" },
        { PANGO_ATTR_ABSOLUTE_LINE_HEIGHT, "PANGO_ATTR_ABSOLUTE_LINE_HEIGHT", "absolute-line-height" },
        { PANGO_ATTR_TEXT_TRANSFORM, "PANGO_ATTR_TEXT_TRANSFORM", "text-transform" },
        { PANGO_ATTR_WORD, "PANGO_ATTR_WORD", "word" },
        { PANGO_ATTR_SENTENCE, "PANGO_ATTR_SENTENCE", "sentence" },
        { PANGO_ATTR_BASELINE_SHIFT, "PANGO_ATTR_BASELINE_SHIFT", "baseline-shift" },
        { PANGO_ATTR_FONT_SCALE, "PANGO_ATTR_FONT_SCALE", "font-scale" },
        { 0, NULL, NULL }
      };
      GType g_define_type_id =
        g_enum_register_static (g_intern_static_string ("PangoAttrType"), values);
      g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
    }

  return g_define_type_id__volatile;
}
GType
pango_underline_get_type (void)
{
  static gsize g_define_type_id__volatile = 0;

  if (g_once_init_enter (&g_define_type_id__volatile))
    {
      static const GEnumValue values[] = {
        { PANGO_UNDERLINE_NONE, "PANGO_UNDERLINE_NONE", "none" },
        { PANGO_UNDERLINE_SINGLE, "PANGO_UNDERLINE_SINGLE", "single" },
        { PANGO_UNDERLINE_DOUBLE, "PANGO_UNDERLINE_DOUBLE", "double" },
        { PANGO_UNDERLINE_LOW, "PANGO_UNDERLINE_LOW", "low" },
        { PANGO_UNDERLINE_ERROR, "PANGO_UNDERLINE_ERROR", "error" },
        { PANGO_UNDERLINE_SINGLE_LINE, "PANGO_UNDERLINE_SINGLE_LINE", "single-line" },
        { PANGO_UNDERLINE_DOUBLE_LINE, "PANGO_UNDERLINE_DOUBLE_LINE", "double-line" },
        { PANGO_UNDERLINE_ERROR_LINE, "PANGO_UNDERLINE_ERROR_LINE", "error-line" },
        { 0, NULL, NULL }
      };
      GType g_define_type_id =
        g_enum_register_static (g_intern_static_string ("PangoUnderline"), values);
      g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
    }

  return g_define_type_id__volatile;
}
GType
pango_overline_get_type (void)
{
  static gsize g_define_type_id__volatile = 0;

  if (g_once_init_enter (&g_define_type_id__volatile))
    {
      static const GEnumValue values[] = {
        { PANGO_OVERLINE_NONE, "PANGO_OVERLINE_NONE", "none" },
        { PANGO_OVERLINE_SINGLE, "PANGO_OVERLINE_SINGLE", "single" },
        { 0, NULL, NULL }
      };
      GType g_define_type_id =
        g_enum_register_static (g_intern_static_string ("PangoOverline"), values);
      g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
    }

  return g_define_type_id__volatile;
}
GType
pango_show_flags_get_type (void)
{
  static gsize g_define_type_id__volatile = 0;

  if (g_once_init_enter (&g_define_type_id__volatile))
    {
      static const GFlagsValue values[] = {
        { PANGO_SHOW_NONE, "PANGO_SHOW_NONE", "none" },
        { PANGO_SHOW_SPACES, "PANGO_SHOW_SPACES", "spaces" },
        { PANGO_SHOW_LINE_BREAKS, "PANGO_SHOW_LINE_BREAKS", "line-breaks" },
        { PANGO_SHOW_IGNORABLES, "PANGO_SHOW_IGNORABLES", "ignorables" },
        { 0, NULL, NULL }
      };
      GType g_define_type_id =
        g_flags_register_static (g_intern_static_string ("PangoShowFlags"), values);
      g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
    }

  return g_define_type_id__volatile;
}
GType
pango_text_transform_get_type (void)
{
  static gsize g_define_type_id__volatile = 0;

  if (g_once_init_enter (&g_define_type_id__volatile))
    {
      static const GEnumValue values[] = {
        { PANGO_TEXT_TRANSFORM_NONE, "PANGO_TEXT_TRANSFORM_NONE", "none" },
        { PANGO_TEXT_TRANSFORM_LOWERCASE, "PANGO_TEXT_TRANSFORM_LOWERCASE", "lowercase" },
        { PANGO_TEXT_TRANSFORM_UPPERCASE, "PANGO_TEXT_TRANSFORM_UPPERCASE", "uppercase" },
        { PANGO_TEXT_TRANSFORM_CAPITALIZE, "PANGO_TEXT_TRANSFORM_CAPITALIZE", "capitalize" },
        { 0, NULL, NULL }
      };
      GType g_define_type_id =
        g_enum_register_static (g_intern_static_string ("PangoTextTransform"), values);
      g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
    }

  return g_define_type_id__volatile;
}
GType
pango_baseline_shift_get_type (void)
{
  static gsize g_define_type_id__volatile = 0;

  if (g_once_init_enter (&g_define_type_id__volatile))
    {
      static const GEnumValue values[] = {
        { PANGO_BASELINE_SHIFT_NONE, "PANGO_BASELINE_SHIFT_NONE", "none" },
        { PANGO_BASELINE_SHIFT_SUPERSCRIPT, "PANGO_BASELINE_SHIFT_SUPERSCRIPT", "superscript" },
        { PANGO_BASELINE_SHIFT_SUBSCRIPT, "PANGO_BASELINE_SHIFT_SUBSCRIPT", "subscript" },
        { 0, NULL, NULL }
      };
      GType g_define_type_id =
        g_enum_register_static (g_intern_static_string ("PangoBaselineShift"), values);
      g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
    }

  return g_define_type_id__volatile;
}
GType
pango_font_scale_get_type (void)
{
  static gsize g_define_type_id__volatile = 0;

  if (g_once_init_enter (&g_define_type_id__volatile))
    {
      static const GEnumValue values[] = {
        { PANGO_FONT_SCALE_NONE, "PANGO_FONT_SCALE_NONE", "none" },
        { PANGO_FONT_SCALE_SUPERSCRIPT, "PANGO_FONT_SCALE_SUPERSCRIPT", "superscript" },
        { PANGO_FONT_SCALE_SUBSCRIPT, "PANGO_FONT_SCALE_SUBSCRIPT", "subscript" },
        { PANGO_FONT_SCALE_SMALL_CAPS, "PANGO_FONT_SCALE_SMALL_CAPS", "small-caps" },
        { 0, NULL, NULL }
      };
      GType g_define_type_id =
        g_enum_register_static (g_intern_static_string ("PangoFontScale"), values);
      g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
    }

  return g_define_type_id__volatile;
}
/* enumerations from "pango-bidi-type.h" */
GType
pango_bidi_type_get_type (void)
{
  static gsize g_define_type_id__volatile = 0;

  if (g_once_init_enter (&g_define_type_id__volatile))
    {
      static const GEnumValue values[] = {
        { PANGO_BIDI_TYPE_L, "PANGO_BIDI_TYPE_L", "l" },
        { PANGO_BIDI_TYPE_LRE, "PANGO_BIDI_TYPE_LRE", "lre" },
        { PANGO_BIDI_TYPE_LRO, "PANGO_BIDI_TYPE_LRO", "lro" },
        { PANGO_BIDI_TYPE_R, "PANGO_BIDI_TYPE_R", "r" },
        { PANGO_BIDI_TYPE_AL, "PANGO_BIDI_TYPE_AL", "al" },
        { PANGO_BIDI_TYPE_RLE, "PANGO_BIDI_TYPE_RLE", "rle" },
        { PANGO_BIDI_TYPE_RLO, "PANGO_BIDI_TYPE_RLO", "rlo" },
        { PANGO_BIDI_TYPE_PDF, "PANGO_BIDI_TYPE_PDF", "pdf" },
        { PANGO_BIDI_TYPE_EN, "PANGO_BIDI_TYPE_EN", "en" },
        { PANGO_BIDI_TYPE_ES, "PANGO_BIDI_TYPE_ES", "es" },
        { PANGO_BIDI_TYPE_ET, "PANGO_BIDI_TYPE_ET", "et" },
        { PANGO_BIDI_TYPE_AN, "PANGO_BIDI_TYPE_AN", "an" },
        { PANGO_BIDI_TYPE_CS, "PANGO_BIDI_TYPE_CS", "cs" },
        { PANGO_BIDI_TYPE_NSM, "PANGO_BIDI_TYPE_NSM", "nsm" },
        { PANGO_BIDI_TYPE_BN, "PANGO_BIDI_TYPE_BN", "bn" },
        { PANGO_BIDI_TYPE_B, "PANGO_BIDI_TYPE_B", "b" },
        { PANGO_BIDI_TYPE_S, "PANGO_BIDI_TYPE_S", "s" },
        { PANGO_BIDI_TYPE_WS, "PANGO_BIDI_TYPE_WS", "ws" },
        { PANGO_BIDI_TYPE_ON, "PANGO_BIDI_TYPE_ON", "on" },
        { PANGO_BIDI_TYPE_LRI, "PANGO_BIDI_TYPE_LRI", "lri" },
        { PANGO_BIDI_TYPE_RLI, "PANGO_BIDI_TYPE_RLI", "rli" },
        { PANGO_BIDI_TYPE_FSI, "PANGO_BIDI_TYPE_FSI", "fsi" },
        { PANGO_BIDI_TYPE_PDI, "PANGO_BIDI_TYPE_PDI", "pdi" },
        { 0, NULL, NULL }
      };
      GType g_define_type_id =
        g_enum_register_static (g_intern_static_string ("PangoBidiType"), values);
      g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
    }

  return g_define_type_id__volatile;
}
/* enumerations from "pango-coverage.h" */
GType
pango_coverage_level_get_type (void)
{
  static gsize g_define_type_id__volatile = 0;

  if (g_once_init_enter (&g_define_type_id__volatile))
    {
      static const GEnumValue values[] = {
        { PANGO_COVERAGE_NONE, "PANGO_COVERAGE_NONE", "none" },
        { PANGO_COVERAGE_FALLBACK, "PANGO_COVERAGE_FALLBACK", "fallback" },
        { PANGO_COVERAGE_APPROXIMATE, "PANGO_COVERAGE_APPROXIMATE", "approximate" },
        { PANGO_COVERAGE_EXACT, "PANGO_COVERAGE_EXACT", "exact" },
        { 0, NULL, NULL }
      };
      GType g_define_type_id =
        g_enum_register_static (g_intern_static_string ("PangoCoverageLevel"), values);
      g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
    }

  return g_define_type_id__volatile;
}
/* enumerations from "pango-direction.h" */
GType
pango_direction_get_type (void)
{
  static gsize g_define_type_id__volatile = 0;

  if (g_once_init_enter (&g_define_type_id__volatile))
    {
      static const GEnumValue values[] = {
        { PANGO_DIRECTION_LTR, "PANGO_DIRECTION_LTR", "ltr" },
        { PANGO_DIRECTION_RTL, "PANGO_DIRECTION_RTL", "rtl" },
        { PANGO_DIRECTION_TTB_LTR, "PANGO_DIRECTION_TTB_LTR", "ttb-ltr" },
        { PANGO_DIRECTION_TTB_RTL, "PANGO_DIRECTION_TTB_RTL", "ttb-rtl" },
        { PANGO_DIRECTION_WEAK_LTR, "PANGO_DIRECTION_WEAK_LTR", "weak-ltr" },
        { PANGO_DIRECTION_WEAK_RTL, "PANGO_DIRECTION_WEAK_RTL", "weak-rtl" },
        { PANGO_DIRECTION_NEUTRAL, "PANGO_DIRECTION_NEUTRAL", "neutral" },
        { 0, NULL, NULL }
      };
      GType g_define_type_id =
        g_enum_register_static (g_intern_static_string ("PangoDirection"), values);
      g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
    }

  return g_define_type_id__volatile;
}
/* enumerations from "pango-font.h" */
GType
pango_style_get_type (void)
{
  static gsize g_define_type_id__volatile = 0;

  if (g_once_init_enter (&g_define_type_id__volatile))
    {
      static const GEnumValue values[] = {
        { PANGO_STYLE_NORMAL, "PANGO_STYLE_NORMAL", "normal" },
        { PANGO_STYLE_OBLIQUE, "PANGO_STYLE_OBLIQUE", "oblique" },
        { PANGO_STYLE_ITALIC, "PANGO_STYLE_ITALIC", "italic" },
        { 0, NULL, NULL }
      };
      GType g_define_type_id =
        g_enum_register_static (g_intern_static_string ("PangoStyle"), values);
      g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
    }

  return g_define_type_id__volatile;
}
GType
pango_variant_get_type (void)
{
  static gsize g_define_type_id__volatile = 0;

  if (g_once_init_enter (&g_define_type_id__volatile))
    {
      static const GEnumValue values[] = {
        { PANGO_VARIANT_NORMAL, "PANGO_VARIANT_NORMAL", "normal" },
        { PANGO_VARIANT_SMALL_CAPS, "PANGO_VARIANT_SMALL_CAPS", "small-caps" },
        { PANGO_VARIANT_ALL_SMALL_CAPS, "PANGO_VARIANT_ALL_SMALL_CAPS", "all-small-caps" },
        { PANGO_VARIANT_PETITE_CAPS, "PANGO_VARIANT_PETITE_CAPS", "petite-caps" },
        { PANGO_VARIANT_ALL_PETITE_CAPS, "PANGO_VARIANT_ALL_PETITE_CAPS", "all-petite-caps" },
        { PANGO_VARIANT_UNICASE, "PANGO_VARIANT_UNICASE", "unicase" },
        { PANGO_VARIANT_TITLE_CAPS, "PANGO_VARIANT_TITLE_CAPS", "title-caps" },
        { 0, NULL, NULL }
      };
      GType g_define_type_id =
        g_enum_register_static (g_intern_static_string ("PangoVariant"), values);
      g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
    }

  return g_define_type_id__volatile;
}
GType
pango_weight_get_type (void)
{
  static gsize g_define_type_id__volatile = 0;

  if (g_once_init_enter (&g_define_type_id__volatile))
    {
      static const GEnumValue values[] = {
        { PANGO_WEIGHT_THIN, "PANGO_WEIGHT_THIN", "thin" },
        { PANGO_WEIGHT_ULTRALIGHT, "PANGO_WEIGHT_ULTRALIGHT", "ultralight" },
        { PANGO_WEIGHT_LIGHT, "PANGO_WEIGHT_LIGHT", "light" },
        { PANGO_WEIGHT_SEMILIGHT, "PANGO_WEIGHT_SEMILIGHT", "semilight" },
        { PANGO_WEIGHT_BOOK, "PANGO_WEIGHT_BOOK", "book" },
        { PANGO_WEIGHT_NORMAL, "PANGO_WEIGHT_NORMAL", "normal" },
        { PANGO_WEIGHT_MEDIUM, "PANGO_WEIGHT_MEDIUM", "medium" },
        { PANGO_WEIGHT_SEMIBOLD, "PANGO_WEIGHT_SEMIBOLD", "semibold" },
        { PANGO_WEIGHT_BOLD, "PANGO_WEIGHT_BOLD", "bold" },
        { PANGO_WEIGHT_ULTRABOLD, "PANGO_WEIGHT_ULTRABOLD", "ultrabold" },
        { PANGO_WEIGHT_HEAVY, "PANGO_WEIGHT_HEAVY", "heavy" },
        { PANGO_WEIGHT_ULTRAHEAVY, "PANGO_WEIGHT_ULTRAHEAVY", "ultraheavy" },
        { 0, NULL, NULL }
      };
      GType g_define_type_id =
        g_enum_register_static (g_intern_static_string ("PangoWeight"), values);
      g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
    }

  return g_define_type_id__volatile;
}
GType
pango_stretch_get_type (void)
{
  static gsize g_define_type_id__volatile = 0;

  if (g_once_init_enter (&g_define_type_id__volatile))
    {
      static const GEnumValue values[] = {
        { PANGO_STRETCH_ULTRA_CONDENSED, "PANGO_STRETCH_ULTRA_CONDENSED", "ultra-condensed" },
        { PANGO_STRETCH_EXTRA_CONDENSED, "PANGO_STRETCH_EXTRA_CONDENSED", "extra-condensed" },
        { PANGO_STRETCH_CONDENSED, "PANGO_STRETCH_CONDENSED", "condensed" },
        { PANGO_STRETCH_SEMI_CONDENSED, "PANGO_STRETCH_SEMI_CONDENSED", "semi-condensed" },
        { PANGO_STRETCH_NORMAL, "PANGO_STRETCH_NORMAL", "normal" },
        { PANGO_STRETCH_SEMI_EXPANDED, "PANGO_STRETCH_SEMI_EXPANDED", "semi-expanded" },
        { PANGO_STRETCH_EXPANDED, "PANGO_STRETCH_EXPANDED", "expanded" },
        { PANGO_STRETCH_EXTRA_EXPANDED, "PANGO_STRETCH_EXTRA_EXPANDED", "extra-expanded" },
        { PANGO_STRETCH_ULTRA_EXPANDED, "PANGO_STRETCH_ULTRA_EXPANDED", "ultra-expanded" },
        { 0, NULL, NULL }
      };
      GType g_define_type_id =
        g_enum_register_static (g_intern_static_string ("PangoStretch"), values);
      g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
    }

  return g_define_type_id__volatile;
}
GType
pango_font_mask_get_type (void)
{
  static gsize g_define_type_id__volatile = 0;

  if (g_once_init_enter (&g_define_type_id__volatile))
    {
      static const GFlagsValue values[] = {
        { PANGO_FONT_MASK_FAMILY, "PANGO_FONT_MASK_FAMILY", "family" },
        { PANGO_FONT_MASK_STYLE, "PANGO_FONT_MASK_STYLE", "style" },
        { PANGO_FONT_MASK_VARIANT, "PANGO_FONT_MASK_VARIANT", "variant" },
        { PANGO_FONT_MASK_WEIGHT, "PANGO_FONT_MASK_WEIGHT", "weight" },
        { PANGO_FONT_MASK_STRETCH, "PANGO_FONT_MASK_STRETCH", "stretch" },
        { PANGO_FONT_MASK_SIZE, "PANGO_FONT_MASK_SIZE", "size" },
        { PANGO_FONT_MASK_GRAVITY, "PANGO_FONT_MASK_GRAVITY", "gravity" },
        { PANGO_FONT_MASK_VARIATIONS, "PANGO_FONT_MASK_VARIATIONS", "variations" },
        { 0, NULL, NULL }
      };
      GType g_define_type_id =
        g_flags_register_static (g_intern_static_string ("PangoFontMask"), values);
      g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
    }

  return g_define_type_id__volatile;
}
/* enumerations from "pango-glyph.h" */
GType
pango_shape_flags_get_type (void)
{
  static gsize g_define_type_id__volatile = 0;

  if (g_once_init_enter (&g_define_type_id__volatile))
    {
      static const GFlagsValue values[] = {
        { PANGO_SHAPE_NONE, "PANGO_SHAPE_NONE", "none" },
        { PANGO_SHAPE_ROUND_POSITIONS, "PANGO_SHAPE_ROUND_POSITIONS", "round-positions" },
        { 0, NULL, NULL }
      };
      GType g_define_type_id =
        g_flags_register_static (g_intern_static_string ("PangoShapeFlags"), values);
      g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
    }

  return g_define_type_id__volatile;
}
/* enumerations from "pango-gravity.h" */
GType
pango_gravity_get_type (void)
{
  static gsize g_define_type_id__volatile = 0;

  if (g_once_init_enter (&g_define_type_id__volatile))
    {
      static const GEnumValue values[] = {
        { PANGO_GRAVITY_SOUTH, "PANGO_GRAVITY_SOUTH", "south" },
        { PANGO_GRAVITY_EAST, "PANGO_GRAVITY_EAST", "east" },
        { PANGO_GRAVITY_NORTH, "PANGO_GRAVITY_NORTH", "north" },
        { PANGO_GRAVITY_WEST, "PANGO_GRAVITY_WEST", "west" },
        { PANGO_GRAVITY_AUTO, "PANGO_GRAVITY_AUTO", "auto" },
        { 0, NULL, NULL }
      };
      GType g_define_type_id =
        g_enum_register_static (g_intern_static_string ("PangoGravity"), values);
      g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
    }

  return g_define_type_id__volatile;
}
GType
pango_gravity_hint_get_type (void)
{
  static gsize g_define_type_id__volatile = 0;

  if (g_once_init_enter (&g_define_type_id__volatile))
    {
      static const GEnumValue values[] = {
        { PANGO_GRAVITY_HINT_NATURAL, "PANGO_GRAVITY_HINT_NATURAL", "natural" },
        { PANGO_GRAVITY_HINT_STRONG, "PANGO_GRAVITY_HINT_STRONG", "strong" },
        { PANGO_GRAVITY_HINT_LINE, "PANGO_GRAVITY_HINT_LINE", "line" },
        { 0, NULL, NULL }
      };
      GType g_define_type_id =
        g_enum_register_static (g_intern_static_string ("PangoGravityHint"), values);
      g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
    }

  return g_define_type_id__volatile;
}
/* enumerations from "pango-layout.h" */
GType
pango_alignment_get_type (void)
{
  static gsize g_define_type_id__volatile = 0;

  if (g_once_init_enter (&g_define_type_id__volatile))
    {
      static const GEnumValue values[] = {
        { PANGO_ALIGN_LEFT, "PANGO_ALIGN_LEFT", "left" },
        { PANGO_ALIGN_CENTER, "PANGO_ALIGN_CENTER", "center" },
        { PANGO_ALIGN_RIGHT, "PANGO_ALIGN_RIGHT", "right" },
        { 0, NULL, NULL }
      };
      GType g_define_type_id =
        g_enum_register_static (g_intern_static_string ("PangoAlignment"), values);
      g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
    }

  return g_define_type_id__volatile;
}
GType
pango_wrap_mode_get_type (void)
{
  static gsize g_define_type_id__volatile = 0;

  if (g_once_init_enter (&g_define_type_id__volatile))
    {
      static const GEnumValue values[] = {
        { PANGO_WRAP_WORD, "PANGO_WRAP_WORD", "word" },
        { PANGO_WRAP_CHAR, "PANGO_WRAP_CHAR", "char" },
        { PANGO_WRAP_WORD_CHAR, "PANGO_WRAP_WORD_CHAR", "word-char" },
        { 0, NULL, NULL }
      };
      GType g_define_type_id =
        g_enum_register_static (g_intern_static_string ("PangoWrapMode"), values);
      g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
    }

  return g_define_type_id__volatile;
}
GType
pango_ellipsize_mode_get_type (void)
{
  static gsize g_define_type_id__volatile = 0;

  if (g_once_init_enter (&g_define_type_id__volatile))
    {
      static const GEnumValue values[] = {
        { PANGO_ELLIPSIZE_NONE, "PANGO_ELLIPSIZE_NONE", "none" },
        { PANGO_ELLIPSIZE_START, "PANGO_ELLIPSIZE_START", "start" },
        { PANGO_ELLIPSIZE_MIDDLE, "PANGO_ELLIPSIZE_MIDDLE", "middle" },
        { PANGO_ELLIPSIZE_END, "PANGO_ELLIPSIZE_END", "end" },
        { 0, NULL, NULL }
      };
      GType g_define_type_id =
        g_enum_register_static (g_intern_static_string ("PangoEllipsizeMode"), values);
      g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
    }

  return g_define_type_id__volatile;
}
GType
pango_layout_serialize_flags_get_type (void)
{
  static gsize g_define_type_id__volatile = 0;

  if (g_once_init_enter (&g_define_type_id__volatile))
    {
      static const GFlagsValue values[] = {
        { PANGO_LAYOUT_SERIALIZE_DEFAULT, "PANGO_LAYOUT_SERIALIZE_DEFAULT", "default" },
        { PANGO_LAYOUT_SERIALIZE_CONTEXT, "PANGO_LAYOUT_SERIALIZE_CONTEXT", "context" },
        { PANGO_LAYOUT_SERIALIZE_OUTPUT, "PANGO_LAYOUT_SERIALIZE_OUTPUT", "output" },
        { 0, NULL, NULL }
      };
      GType g_define_type_id =
        g_flags_register_static (g_intern_static_string ("PangoLayoutSerializeFlags"), values);
      g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
    }

  return g_define_type_id__volatile;
}
GType
pango_layout_deserialize_error_get_type (void)
{
  static gsize g_define_type_id__volatile = 0;

  if (g_once_init_enter (&g_define_type_id__volatile))
    {
      static const GEnumValue values[] = {
        { PANGO_LAYOUT_DESERIALIZE_INVALID, "PANGO_LAYOUT_DESERIALIZE_INVALID", "invalid" },
        { PANGO_LAYOUT_DESERIALIZE_INVALID_VALUE, "PANGO_LAYOUT_DESERIALIZE_INVALID_VALUE", "invalid-value" },
        { PANGO_LAYOUT_DESERIALIZE_MISSING_VALUE, "PANGO_LAYOUT_DESERIALIZE_MISSING_VALUE", "missing-value" },
        { 0, NULL, NULL }
      };
      GType g_define_type_id =
        g_enum_register_static (g_intern_static_string ("PangoLayoutDeserializeError"), values);
      g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
    }

  return g_define_type_id__volatile;
}
GType
pango_layout_deserialize_flags_get_type (void)
{
  static gsize g_define_type_id__volatile = 0;

  if (g_once_init_enter (&g_define_type_id__volatile))
    {
      static const GFlagsValue values[] = {
        { PANGO_LAYOUT_DESERIALIZE_DEFAULT, "PANGO_LAYOUT_DESERIALIZE_DEFAULT", "default" },
        { PANGO_LAYOUT_DESERIALIZE_CONTEXT, "PANGO_LAYOUT_DESERIALIZE_CONTEXT", "context" },
        { 0, NULL, NULL }
      };
      GType g_define_type_id =
        g_flags_register_static (g_intern_static_string ("PangoLayoutDeserializeFlags"), values);
      g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
    }

  return g_define_type_id__volatile;
}
/* enumerations from "pango-renderer.h" */
GType
pango_render_part_get_type (void)
{
  static gsize g_define_type_id__volatile = 0;

  if (g_once_init_enter (&g_define_type_id__volatile))
    {
      static const GEnumValue values[] = {
        { PANGO_RENDER_PART_FOREGROUND, "PANGO_RENDER_PART_FOREGROUND", "foreground" },
        { PANGO_RENDER_PART_BACKGROUND, "PANGO_RENDER_PART_BACKGROUND", "background" },
        { PANGO_RENDER_PART_UNDERLINE, "PANGO_RENDER_PART_UNDERLINE", "underline" },
        { PANGO_RENDER_PART_STRIKETHROUGH, "PANGO_RENDER_PART_STRIKETHROUGH", "strikethrough" },
        { PANGO_RENDER_PART_OVERLINE, "PANGO_RENDER_PART_OVERLINE", "overline" },
        { 0, NULL, NULL }
      };
      GType g_define_type_id =
        g_enum_register_static (g_intern_static_string ("PangoRenderPart"), values);
      g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
    }

  return g_define_type_id__volatile;
}
/* enumerations from "pango-script.h" */
GType
pango_script_get_type (void)
{
  static gsize g_define_type_id__volatile = 0;

  if (g_once_init_enter (&g_define_type_id__volatile))
    {
      static const GEnumValue values[] = {
        { PANGO_SCRIPT_INVALID_CODE, "PANGO_SCRIPT_INVALID_CODE", "invalid-code" },
        { PANGO_SCRIPT_COMMON, "PANGO_SCRIPT_COMMON", "common" },
        { PANGO_SCRIPT_INHERITED, "PANGO_SCRIPT_INHERITED", "inherited" },
        { PANGO_SCRIPT_ARABIC, "PANGO_SCRIPT_ARABIC", "arabic" },
        { PANGO_SCRIPT_ARMENIAN, "PANGO_SCRIPT_ARMENIAN", "armenian" },
        { PANGO_SCRIPT_BENGALI, "PANGO_SCRIPT_BENGALI", "bengali" },
        { PANGO_SCRIPT_BOPOMOFO, "PANGO_SCRIPT_BOPOMOFO", "bopomofo" },
        { PANGO_SCRIPT_CHEROKEE, "PANGO_SCRIPT_CHEROKEE", "cherokee" },
        { PANGO_SCRIPT_COPTIC, "PANGO_SCRIPT_COPTIC", "coptic" },
        { PANGO_SCRIPT_CYRILLIC, "PANGO_SCRIPT_CYRILLIC", "cyrillic" },
        { PANGO_SCRIPT_DESERET, "PANGO_SCRIPT_DESERET", "deseret" },
        { PANGO_SCRIPT_DEVANAGARI, "PANGO_SCRIPT_DEVANAGARI", "devanagari" },
        { PANGO_SCRIPT_ETHIOPIC, "PANGO_SCRIPT_ETHIOPIC", "ethiopic" },
        { PANGO_SCRIPT_GEORGIAN, "PANGO_SCRIPT_GEORGIAN", "georgian" },
        { PANGO_SCRIPT_GOTHIC, "PANGO_SCRIPT_GOTHIC", "gothic" },
        { PANGO_SCRIPT_GREEK, "PANGO_SCRIPT_GREEK", "greek" },
        { PANGO_SCRIPT_GUJARATI, "PANGO_SCRIPT_GUJARATI", "gujarati" },
        { PANGO_SCRIPT_GURMUKHI, "PANGO_SCRIPT_GURMUKHI", "gurmukhi" },
        { PANGO_SCRIPT_HAN, "PANGO_SCRIPT_HAN", "han" },
        { PANGO_SCRIPT_HANGUL, "PANGO_SCRIPT_HANGUL", "hangul" },
        { PANGO_SCRIPT_HEBREW, "PANGO_SCRIPT_HEBREW", "hebrew" },
        { PANGO_SCRIPT_HIRAGANA, "PANGO_SCRIPT_HIRAGANA", "hiragana" },
        { PANGO_SCRIPT_KANNADA, "PANGO_SCRIPT_KANNADA", "kannada" },
        { PANGO_SCRIPT_KATAKANA, "PANGO_SCRIPT_KATAKANA", "katakana" },
        { PANGO_SCRIPT_KHMER, "PANGO_SCRIPT_KHMER", "khmer" },
        { PANGO_SCRIPT_LAO, "PANGO_SCRIPT_LAO", "lao" },
        { PANGO_SCRIPT_LATIN, "PANGO_SCRIPT_LATIN", "latin" },
        { PANGO_SCRIPT_MALAYALAM, "PANGO_SCRIPT_MALAYALAM", "malayalam" },
        { PANGO_SCRIPT_MONGOLIAN, "PANGO_SCRIPT_MONGOLIAN", "mongolian" },
        { PANGO_SCRIPT_MYANMAR, "PANGO_SCRIPT_MYANMAR", "myanmar" },
        { PANGO_SCRIPT_OGHAM, "PANGO_SCRIPT_OGHAM", "ogham" },
        { PANGO_SCRIPT_OLD_ITALIC, "PANGO_SCRIPT_OLD_ITALIC", "old-italic" },
        { PANGO_SCRIPT_ORIYA, "PANGO_SCRIPT_ORIYA", "oriya" },
        { PANGO_SCRIPT_RUNIC, "PANGO_SCRIPT_RUNIC", "runic" },
        { PANGO_SCRIPT_SINHALA, "PANGO_SCRIPT_SINHALA", "sinhala" },
        { PANGO_SCRIPT_SYRIAC, "PANGO_SCRIPT_SYRIAC", "syriac" },
        { PANGO_SCRIPT_TAMIL, "PANGO_SCRIPT_TAMIL", "tamil" },
        { PANGO_SCRIPT_TELUGU, "PANGO_SCRIPT_TELUGU", "telugu" },
        { PANGO_SCRIPT_THAANA, "PANGO_SCRIPT_THAANA", "thaana" },
        { PANGO_SCRIPT_THAI, "PANGO_SCRIPT_THAI", "thai" },
        { PANGO_SCRIPT_TIBETAN, "PANGO_SCRIPT_TIBETAN", "tibetan" },
        { PANGO_SCRIPT_CANADIAN_ABORIGINAL, "PANGO_SCRIPT_CANADIAN_ABORIGINAL", "canadian-aboriginal" },
        { PANGO_SCRIPT_YI, "PANGO_SCRIPT_YI", "yi" },
        { PANGO_SCRIPT_TAGALOG, "PANGO_SCRIPT_TAGALOG", "tagalog" },
        { PANGO_SCRIPT_HANUNOO, "PANGO_SCRIPT_HANUNOO", "hanunoo" },
        { PANGO_SCRIPT_BUHID, "PANGO_SCRIPT_BUHID", "buhid" },
        { PANGO_SCRIPT_TAGBANWA, "PANGO_SCRIPT_TAGBANWA", "tagbanwa" },
        { PANGO_SCRIPT_BRAILLE, "PANGO_SCRIPT_BRAILLE", "braille" },
        { PANGO_SCRIPT_CYPRIOT, "PANGO_SCRIPT_CYPRIOT", "cypriot" },
        { PANGO_SCRIPT_LIMBU, "PANGO_SCRIPT_LIMBU", "limbu" },
        { PANGO_SCRIPT_OSMANYA, "PANGO_SCRIPT_OSMANYA", "osmanya" },
        { PANGO_SCRIPT_SHAVIAN, "PANGO_SCRIPT_SHAVIAN", "shavian" },
        { PANGO_SCRIPT_LINEAR_B, "PANGO_SCRIPT_LINEAR_B", "linear-b" },
        { PANGO_SCRIPT_TAI_LE, "PANGO_SCRIPT_TAI_LE", "tai-le" },
        { PANGO_SCRIPT_UGARITIC, "PANGO_SCRIPT_UGARITIC", "ugaritic" },
        { PANGO_SCRIPT_NEW_TAI_LUE, "PANGO_SCRIPT_NEW_TAI_LUE", "new-tai-lue" },
        { PANGO_SCRIPT_BUGINESE, "PANGO_SCRIPT_BUGINESE", "buginese" },
        { PANGO_SCRIPT_GLAGOLITIC, "PANGO_SCRIPT_GLAGOLITIC", "glagolitic" },
        { PANGO_SCRIPT_TIFINAGH, "PANGO_SCRIPT_TIFINAGH", "tifinagh" },
        { PANGO_SCRIPT_SYLOTI_NAGRI, "PANGO_SCRIPT_SYLOTI_NAGRI", "syloti-nagri" },
        { PANGO_SCRIPT_OLD_PERSIAN, "PANGO_SCRIPT_OLD_PERSIAN", "old-persian" },
        { PANGO_SCRIPT_KHAROSHTHI, "PANGO_SCRIPT_KHAROSHTHI", "kharoshthi" },
        { PANGO_SCRIPT_UNKNOWN, "PANGO_SCRIPT_UNKNOWN", "unknown" },
        { PANGO_SCRIPT_BALINESE, "PANGO_SCRIPT_BALINESE", "balinese" },
        { PANGO_SCRIPT_CUNEIFORM, "PANGO_SCRIPT_CUNEIFORM", "cuneiform" },
        { PANGO_SCRIPT_PHOENICIAN, "PANGO_SCRIPT_PHOENICIAN", "phoenician" },
        { PANGO_SCRIPT_PHAGS_PA, "PANGO_SCRIPT_PHAGS_PA", "phags-pa" },
        { PANGO_SCRIPT_NKO, "PANGO_SCRIPT_NKO", "nko" },
        { PANGO_SCRIPT_KAYAH_LI, "PANGO_SCRIPT_KAYAH_LI", "kayah-li" },
        { PANGO_SCRIPT_LEPCHA, "PANGO_SCRIPT_LEPCHA", "lepcha" },
        { PANGO_SCRIPT_REJANG, "PANGO_SCRIPT_REJANG", "rejang" },
        { PANGO_SCRIPT_SUNDANESE, "PANGO_SCRIPT_SUNDANESE", "sundanese" },
        { PANGO_SCRIPT_SAURASHTRA, "PANGO_SCRIPT_SAURASHTRA", "saurashtra" },
        { PANGO_SCRIPT_CHAM, "PANGO_SCRIPT_CHAM", "cham" },
        { PANGO_SCRIPT_OL_CHIKI, "PANGO_SCRIPT_OL_CHIKI", "ol-chiki" },
        { PANGO_SCRIPT_VAI, "PANGO_SCRIPT_VAI", "vai" },
        { PANGO_SCRIPT_CARIAN, "PANGO_SCRIPT_CARIAN", "carian" },
        { PANGO_SCRIPT_LYCIAN, "PANGO_SCRIPT_LYCIAN", "lycian" },
        { PANGO_SCRIPT_LYDIAN, "PANGO_SCRIPT_LYDIAN", "lydian" },
        { PANGO_SCRIPT_BATAK, "PANGO_SCRIPT_BATAK", "batak" },
        { PANGO_SCRIPT_BRAHMI, "PANGO_SCRIPT_BRAHMI", "brahmi" },
        { PANGO_SCRIPT_MANDAIC, "PANGO_SCRIPT_MANDAIC", "mandaic" },
        { PANGO_SCRIPT_CHAKMA, "PANGO_SCRIPT_CHAKMA", "chakma" },
        { PANGO_SCRIPT_MEROITIC_CURSIVE, "PANGO_SCRIPT_MEROITIC_CURSIVE", "meroitic-cursive" },
        { PANGO_SCRIPT_MEROITIC_HIEROGLYPHS, "PANGO_SCRIPT_MEROITIC_HIEROGLYPHS", "meroitic-hieroglyphs" },
        { PANGO_SCRIPT_MIAO, "PANGO_SCRIPT_MIAO", "miao" },
        { PANGO_SCRIPT_SHARADA, "PANGO_SCRIPT_SHARADA", "sharada" },
        { PANGO_SCRIPT_SORA_SOMPENG, "PANGO_SCRIPT_SORA_SOMPENG", "sora-sompeng" },
        { PANGO_SCRIPT_TAKRI, "PANGO_SCRIPT_TAKRI", "takri" },
        { PANGO_SCRIPT_BASSA_VAH, "PANGO_SCRIPT_BASSA_VAH", "bassa-vah" },
        { PANGO_SCRIPT_CAUCASIAN_ALBANIAN, "PANGO_SCRIPT_CAUCASIAN_ALBANIAN", "caucasian-albanian" },
        { PANGO_SCRIPT_DUPLOYAN, "PANGO_SCRIPT_DUPLOYAN", "duployan" },
        { PANGO_SCRIPT_ELBASAN, "PANGO_SCRIPT_ELBASAN", "elbasan" },
        { PANGO_SCRIPT_GRANTHA, "PANGO_SCRIPT_GRANTHA", "grantha" },
        { PANGO_SCRIPT_KHOJKI, "PANGO_SCRIPT_KHOJKI", "khojki" },
        { PANGO_SCRIPT_KHUDAWADI, "PANGO_SCRIPT_KHUDAWADI", "khudawadi" },
        { PANGO_SCRIPT_LINEAR_A, "PANGO_SCRIPT_LINEAR_A", "linear-a" },
        { PANGO_SCRIPT_MAHAJANI, "PANGO_SCRIPT_MAHAJANI", "mahajani" },
        { PANGO_SCRIPT_MANICHAEAN, "PANGO_SCRIPT_MANICHAEAN", "manichaean" },
        { PANGO_SCRIPT_MENDE_KIKAKUI, "PANGO_SCRIPT_MENDE_KIKAKUI", "mende-kikakui" },
        { PANGO_SCRIPT_MODI, "PANGO_SCRIPT_MODI", "modi" },
        { PANGO_SCRIPT_MRO, "PANGO_SCRIPT_MRO", "mro" },
        { PANGO_SCRIPT_NABATAEAN, "PANGO_SCRIPT_NABATAEAN", "nabataean" },
        { PANGO_SCRIPT_OLD_NORTH_ARABIAN, "PANGO_SCRIPT_OLD_NORTH_ARABIAN", "old-north-arabian" },
        { PANGO_SCRIPT_OLD_PERMIC, "PANGO_SCRIPT_OLD_PERMIC", "old-permic" },
        { PANGO_SCRIPT_PAHAWH_HMONG, "PANGO_SCRIPT_PAHAWH_HMONG", "pahawh-hmong" },
        { PANGO_SCRIPT_PALMYRENE, "PANGO_SCRIPT_PALMYRENE", "palmyrene" },
        { PANGO_SCRIPT_PAU_CIN_HAU, "PANGO_SCRIPT_PAU_CIN_HAU", "pau-cin-hau" },
        { PANGO_SCRIPT_PSALTER_PAHLAVI, "PANGO_SCRIPT_PSALTER_PAHLAVI", "psalter-pahlavi" },
        { PANGO_SCRIPT_SIDDHAM, "PANGO_SCRIPT_SIDDHAM", "siddham" },
        { PANGO_SCRIPT_TIRHUTA, "PANGO_SCRIPT_TIRHUTA", "tirhuta" },
        { PANGO_SCRIPT_WARANG_CITI, "PANGO_SCRIPT_WARANG_CITI", "warang-citi" },
        { PANGO_SCRIPT_AHOM, "PANGO_SCRIPT_AHOM", "ahom" },
        { PANGO_SCRIPT_ANATOLIAN_HIEROGLYPHS, "PANGO_SCRIPT_ANATOLIAN_HIEROGLYPHS", "anatolian-hieroglyphs" },
        { PANGO_SCRIPT_HATRAN, "PANGO_SCRIPT_HATRAN", "hatran" },
        { PANGO_SCRIPT_MULTANI, "PANGO_SCRIPT_MULTANI", "multani" },
        { PANGO_SCRIPT_OLD_HUNGARIAN, "PANGO_SCRIPT_OLD_HUNGARIAN", "old-hungarian" },
        { PANGO_SCRIPT_SIGNWRITING, "PANGO_SCRIPT_SIGNWRITING", "signwriting" },
        { 0, NULL, NULL }
      };
      GType g_define_type_id =
        g_enum_register_static (g_intern_static_string ("PangoScript"), values);
      g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
    }

  return g_define_type_id__volatile;
}
/* enumerations from "pango-tabs.h" */
GType
pango_tab_align_get_type (void)
{
  static gsize g_define_type_id__volatile = 0;

  if (g_once_init_enter (&g_define_type_id__volatile))
    {
      static const GEnumValue values[] = {
        { PANGO_TAB_LEFT, "PANGO_TAB_LEFT", "left" },
        { PANGO_TAB_RIGHT, "PANGO_TAB_RIGHT", "right" },
        { PANGO_TAB_CENTER, "PANGO_TAB_CENTER", "center" },
        { PANGO_TAB_DECIMAL, "PANGO_TAB_DECIMAL", "decimal" },
        { 0, NULL, NULL }
      };
      GType g_define_type_id =
        g_enum_register_static (g_intern_static_string ("PangoTabAlign"), values);
      g_once_init_leave (&g_define_type_id__volatile, g_define_type_id);
    }

  return g_define_type_id__volatile;
}

/* Generated data ends here */

